"""
Testing suite for ws_metrics
"""
import argparse
import logging
import sys
from unittest.mock import patch

import pytest
from ws_metrics.deep_thought import LifeUniverseEverything

logger = logging.getLogger("test_ws_metrics")
logger.setLevel(logging.INFO)
logger.addHandler(logging.StreamHandler(sys.stdout))

args_b = argparse.Namespace(between=['null', '2021-03-30T00:00:00', '2021-03-30T23:59:59'], capability=None)
result_b = 4

args_c = argparse.Namespace(between=None, capability='null')
result_c = 10


def mock_leu(args: argparse.Namespace) -> LifeUniverseEverything:
    with patch("psycopg2.connect") as mock_connect:
        return LifeUniverseEverything(connection=mock_connect, args=args)


mock_leu_b = mock_leu(args_b)
mock_leu_c = mock_leu(args_c)


class TestWSMetrics:

    def test_get_total_cap_executions(self):
        mock_leu_c.conn.cursor.return_value.fetchone.return_value = result_c
        assert args_c.capability == 'null'
        value = mock_leu_c.get_total_cap_executions(args_c.capability)
        assert value == result_c

    def test_get_total_executions_in_range(self):
        mock_leu_b.conn.cursor.return_value.fetchone.return_value = result_b
        assert args_b.between[0] == 'null'
        assert args_b.between[1] == '2021-03-30T00:00:00'
        assert args_b.between[2] == '2021-03-30T23:59:59'
        value = mock_leu_b.get_total_executions_in_range(args_b.between[0],
                                                         args_b.between[1],
                                                         args_b.between[2])
        assert value == result_b
