# Metrics make the world go round.
# Yes, this is a reference to Hitchhikers Guide to the Galaxy.

"""
scripted access to the metrics database views for data analysts
"""

import argparse
import datetime
import pprint
from typing import Dict

import psycopg2
import psycopg2.extras
import logging
import sys
from pycapo import CapoConfig

logger = logging.getLogger("ws_metrics")
logger.setLevel(logging.INFO)
logger.addHandler(logging.StreamHandler(sys.stdout))


class LifeUniverseEverything:

    def __init__(self, connection, args: argparse.Namespace):
        self.conn = connection
        self.args = args

    def get_total_cap_executions(self, capability_name: str):
        sql = "SELECT count(*) From capability_execution_metrics WHERE cap_name = \'" + capability_name[0] + "\'"
        cursor = self.conn.cursor()
        cursor.execute(sql)
        total_number = cursor.fetchone()
        print("SQL query used: " + sql)
        return total_number

    def get_total_executions_in_range(self,
                                      capability_name: str,
                                      beginning_date: datetime,
                                      end_date: datetime):
        sql = "SELECT count(*) FROM capability_execution_metrics WHERE cap_name = \'" + capability_name +\
              "\' AND execution_created between \'" + beginning_date +\
              "\' and \'" + end_date + "\'"
        cursor = self.conn.cursor()
        cursor.execute(sql)
        total_number = cursor.fetchone()
        print("SQL query used: " + sql)
        return total_number

    def run_metrics(self, args: argparse.Namespace):
        if args.capability:
            result = self.get_total_cap_executions(args.capability)
            print("\nTotal number of " + args.capability[0] + " capability executions: " + str(result[0]) + "\n")
        if args.between:
            result = self.get_total_executions_in_range(args.between[0], args.between[1], args.between[2])
            print("\nTotal number of " + args.between[0] +
                  " executions between " + args.between[1] +
                  " and " + args.between[2] + ": " + str(result[0]) + "\n")


def parser() -> argparse.ArgumentParser:
    """
    Initialize argument parser for command-line arguments

    :return: Argument parser
    """
    parser = argparse.ArgumentParser(
        description="Workspaces metrics reporter",
        formatter_class=argparse.RawTextHelpFormatter
    )
    parser.add_argument(
        "-c",
        "--capability",
        nargs=1,
        action="store",
        required=False,
        help="report number of times a capability has executed"
    )
    parser.add_argument(
        "-b",
        "--between",
        nargs=3,
        action="store",
        required=False,
        help="report the number of times a capability has executed between two user provided timestamps."
    )

    return parser


def _get_capo_settings():
    return CapoConfig().settings("metadataDatabase")


def main():
    print("**********************************")
    print("* WELCOME TO WORKSPACES METRICS! *")
    print("**********************************")
    args = parser().parse_args()
    settings = _get_capo_settings()
    host, dbname = settings['jdbcUrl'].split(':')[2][2:].split('/')
    try:
        conn = psycopg2.connect(
            host=host,
            database=dbname,
            user=settings.jdbcUsername,
            password=settings.jdbcPassword
        )
    except:
        print("Unable to connect to database.")

    LifeUniverseEverything(conn, args).run_metrics(args)
    conn.close()

